<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

<!--
  Purpose:  
  Convert a result set into a docbook article containing a single section with a single table.

  Description:
  With a "raw" resultSet you get a table with a title of "Results" with as many columns as
  you have columns in the resultSet. The title of each column is the same as the name of the
  column in the select statement that produced the result set. Each column is 1in wide.

  You can modify 

  1. The title of the article. To do this add an attribute called title to the result set.
     This can be done using twset. An example is 

     twset {priorityList/resultSet/@title} "Attribute Title"

     where the resultSet is in a variable called priorityList and the new title will be
     "Attribute Title".

  2. The title of each column. You may find it easier to modify this by changing the name
     retrieved in the select statement than by using this technique. To modify the title add
     a title attribute to the column element of the column you want to provide a title for in
     the first record. For example 

     twset {priorityList/resultSet/record[1]/column[1]/@title} "Priority"

     Note the use of record[1] to identify the first record. This is important as only the
     first record is examined when setting the titles for the columns.

  3. The width of each column. To do this add a colwidth attribute to the column element you want
     to set the width of in the first record. For example

     twset {priorityList/resultSet/record[1]/column[1]/@colwidth} "0.5in"

  A complete example script is 

  twset {priorityList/resultSet/@title} "Attribute Title"
  twset {priorityList/resultSet/record[1]/column[1]/@title} "Priority"
  twset {priorityList/resultSet/record[1]/column[1]/@colwidth} "0.5in"
  twset {priorityList/resultSet/record[1]/column[2]/@title} "Description"
  twset docbook [twtransform priorityList "resultSetToDocbook.xsl"]

  The script sets the title of the whole article to "Attribute Title", the title of
  the first column to "Priority" and the title of the second column to "Description". It 
  also sets the width of the first column to 0.5in. 
 
  The call to twtransform applies this transform ("resultSetToDocbook.xsl") to the priorityList
  and stores the results in a variable called docbook.

  Display as PDF:
  To display the docbook variable as pdf you need to use the invoke the xml2pdf servlet. The iframe 
  example tag below shows how to do this. The name "something.pdf" is not relevant, the pdf data
  is generated automatically. The zTaskId parameter is needed to identify the task whose variables
  are to be used. zXMLParamName provides the name of the variable that contains the xml to convert
  to pdf. In this example it is set to docbook, which matches the name of the variable set in the
  example above. zXSLFileName provides the name of the transformation that is will convert the 
  contents of the docbook variable to pdf. The final parameter, "x.pdf", is needed to convince
  internet explorer that it is actually going to receive pdf data.

  <iframe src="something.pdf?zTaskId={{tw_task_id}}&zXMLParamName=docbook&zXSLFileName=docbook/fo/docbook.xsl&x.pdf" width="80%" height="100%">
  </iframe>

  Installation:
  In the 50AppServer.xml file you will see an element XSLDefaultScriptLocation the value of
  which identifies a directory. This file ("resultSetToDocbook.xsl") must be placed in that 
  directory. Also in that directory you must have a subdirectory called docbook that contains
  the docbook xsl files.

  Note the doctype-system value in the xsl:output statement below. This is correct for executing on
  websphere. It provides the relative path to the sdocbook.dtd from the location where the appserver
  is executed. For weblogic it should be "./resources/script/docbook/sdocbook.dtd"

-->

<xsl:import href="resultSetCommon.xsl"/>

<xsl:output method="xml" version="1.0" omit-xml-declaration="no"
            standalone="no" indent="yes" doctype-system="../lib/teamworks/resources/script/docbook/sdocbook.dtd"
            doctype-public="http://www.oasis-open.org/docbook/xml/simple/4.1.2.5/sdocbook.dtd"/>

<xsl:template match="/resultSet">
  <article>
    <title>
      <xsl:call-template name="resultSetTitle"/>
    </title>
    <section>
      <informaltable frame="none">
        <tgroup>
           <xsl:attribute name="cols"><xsl:value-of select="@columnCount"/></xsl:attribute>
           <xsl:apply-templates select="record[1]/column" mode="colspec"/>           
           <thead>
             <row>
               <xsl:apply-templates select="record[1]/column" mode="header"/>
             </row>
           </thead>
           <tbody>
             <xsl:apply-templates select="record" mode="body"/>
           </tbody>
        </tgroup>
      </informaltable>
    </section>
  </article>
</xsl:template>

<xsl:template match="record" mode="body">
  <row>
    <xsl:apply-templates select="column" mode="body"/>
  </row>
</xsl:template>

<xsl:template match="column" mode="colspec">
  <colspec>
    <xsl:attribute name="colsep">1</xsl:attribute>
    <xsl:attribute name="colname"><xsl:value-of select="@name"/></xsl:attribute>
    <xsl:attribute name="align">left</xsl:attribute>
    <xsl:attribute name="colwidth">
      <xsl:choose>
        <xsl:when test="@colwidth"><xsl:value-of select="@colwidth"/></xsl:when>
        <xsl:otherwise>1in</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </colspec>
</xsl:template>

<xsl:template match="column" mode="header">
  <entry>
    <xsl:attribute name="colname"><xsl:value-of select="@name"/></xsl:attribute>
    <para>
      <xsl:call-template name="columnTitle"/>
    </para>
  </entry>
</xsl:template>

<xsl:template match="column" mode="body">
  <entry>
    <xsl:attribute name="colname"><xsl:value-of select="@name"/></xsl:attribute>
    <para><xsl:value-of select="."/></para>
  </entry>
</xsl:template>

</xsl:stylesheet>