<?xml version="1.0"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">
<!--
  Purpose:
  Convert a result set into an html table.

  Description: With a "raw" result set you get an html table element
  with as many columns as you have columns in the resultSet. The title
  of each column is the same as the name of the column in the select
  statement that produced the result set.

  You can modify

  1. Whether or not the table element is returned alone or is returned
     as part of an html page. To get an html page add the attribute
     htmlPageWanted with a value of true to the resultSetElement. You
     can do this using twset. An example, assuming the resultSet is
     contained in a variable called priorityList, is

     twset {priorityList/resultSet/@htmlPageWanted} "true"

  2. If you have chosen to generate an html page, by setting the
     htmlPageWanted attribute, you can also set the title for the
     page. If you don't set a title then a default title of "Results"
     is used. To set a title add an attribute called title to the
     resultSet element. For example
     
     twset {priorityList/resultSet/@title} "Attribute Title"

     where the resultSet is in a variable called priorityList and the
     new title will be "Attribute Title".

  3. The title of each column. You may find it easier to modify this
     by changing the name retrieved in the select statement than by
     using this technique. To modify the title add a title attribute
     to the column element of the column you want to provide a title
     for in the first record. For example

     twset {priorityList/resultSet/record[1]/column[1]/@title} "Priority"

     Note the use of record[1] to identify the first record. This is
     important as only the first record is examined when setting the
     titles for the columns.

  4. Normally you get back a single row of column titles at the top of
     the table. By adding a groupSize attribute to the resultSet
     element you can choose to have more than one header row. The
     groupSize is the maximum number of records that are output after
     each header row.

     twset {priorityList/resultSet/@groupSize} "3"
-->
  
<!-- Import the common templates -->
<xsl:import href="resultSetCommon.xsl"/>

<xsl:output method="html"/>

<!-- If an html page is wanted add an attribute called htmlPageWanted -->
<!-- with a value of "true" to the resultSet element. Having an html -->
<!-- page means you get <html><head><title>...</title></head><body>...</body></html> -->
<!-- wrapped round your table. If you don't have the attribute you just get -->
<!-- the table. -->
<xsl:variable name="htmlPageWanted">
  <xsl:choose>
    <xsl:when test="resultSet/@htmlPageWanted">
      <xsl:choose>
        <xsl:when test="resultSet/@htmlPageWanted = 'true'">true</xsl:when>
        <xsl:otherwise>false</xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>false</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- The total number of records in the result set. -->
<xsl:variable name="count" select="count(resultSet/record)"/>

<!-- The size of the group to use for grouping. -->
<!-- This is taken from the groupSize attribute on the -->
<!-- resultSet element. If there is no groupSize attribute -->
<!-- then a groupSize equal to the total number of records -->
<!-- is used. This means you get one group. -->
<xsl:variable name="groupSize">
  <xsl:choose>
    <xsl:when test="resultSet/@groupSize">
      <xsl:value-of select="resultSet/@groupSize"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$count"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<!-- The zero based index of the first record to display -->
<xsl:variable name="startingPosition" select="0"/>
    
<!-- The entry point template. It calls htmlPage or table -->
<!-- depending on the value of the htmlPageWanted variable -->
<xsl:template match="resultSet">
  <xsl:choose>
    <xsl:when test="$htmlPageWanted = 'true'">
      <xsl:call-template name="htmlPage"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="table"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Output a complete html page -->
<xsl:template name="htmlPage">
  <html>
    <head>
	  <title>
        <xsl:call-template name="resultSetTitle"/>
      </title>
	</head>
	<body>
      <xsl:call-template name="table"/>
	</body>
  </html>
</xsl:template>

<!-- Output an html table. Use the recurse template to do grouping -->
<xsl:template name="table">
  <table>
    <xsl:call-template name="recurse">
      <xsl:with-param name="count" select="$count"/>
      <xsl:with-param name="groupSize" select="$groupSize"/>
      <xsl:with-param name="currentPosition" select="$startingPosition"/>
    </xsl:call-template>
  </table>
</xsl:template>

<!--
  This is the template that actually does the grouping.
  It repeatedly calls the template called processGroup each
  time passing it different values for from and to,
  the parameters that limit the records processed in each call.
  This template is independent of the xml structure being
  processed, all it cares about are the parameters it is passed.
-->
<xsl:template name="recurse">
  <xsl:param name="count"/>
  <xsl:param name="groupSize"/>
  <xsl:param name="currentPosition"/>
  <!-- If there are still records to process-->
  <xsl:if test="$currentPosition &lt; $count">
    <!-- Work out the new endPosition    -->
    <xsl:variable name="endPosition" select="$currentPosition + $groupSize"/>
    <!-- Process the group from currentPostion up to endPosition-->
    <xsl:call-template name="processGroup">
      <xsl:with-param name="from" select="$currentPosition"/>
      <xsl:with-param name="to" select="$endPosition"/>
    </xsl:call-template>
    <!-- Now call ourselves to process any remaining records-->
    <xsl:call-template name="recurse">
      <xsl:with-param name="count" select="$count"/>
      <xsl:with-param name="groupSize" select="$groupSize"/>
      <xsl:with-param name="currentPosition" select="$endPosition"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!--
  The processGroup template calls the appropriate templates
  with the selected set of nodes being those with positions
  greater than the "from" parameter and less than or equal to
  the "to" parameter.
-->
<xsl:template name="processGroup">
  <xsl:param name="from"/>
  <xsl:param name="to"/>
  <!-- Some header info-->
  <xsl:apply-templates select="record[1]" mode="header"/>
  <!-- Templates matching "record" with a mode of body will be called for each record in the group-->
  <xsl:apply-templates select="record[position() > $from and position() &lt;= $to]" mode="body"/>
  
</xsl:template>

<!-- Output a header row -->
<xsl:template match="record" mode="header">
  <tr>
    <xsl:apply-templates select="column" mode="header"/>
  </tr>
</xsl:template>

<!-- Output a body row -->
<xsl:template match="record" mode="body">
  <tr>
	<xsl:apply-templates select="column" mode="body"/>
  </tr>
</xsl:template>

<!-- Output the header for a column -->
<xsl:template match="column" mode="header">
  <td>
	<b><xsl:call-template name="columnTitle"/></b>
  </td>
</xsl:template>

<!-- Output the data for a single column in a row -->
<xsl:template match="column" mode="body">
  <td>
	<xsl:value-of select="." />
  </td>
</xsl:template>

</xsl:stylesheet>